// --------------- note setup ----------------------------------

// 12화음 이름 정의
typedef enum {
  NOTE_C, NOTE_Cs, NOTE_D, NOTE_Eb, NOTE_E, NOTE_F, NOTE_Fs, NOTE_G, NOTE_Gs, 
  NOTE_A, NOTE_Bb, NOTE_B, NOTE_MAX
} note_t;

// 화음별 피에조 부저 진동수 정의 및 옥타브에 따른 값 변경 함수
double ledcWriteNote(note_t note, uint8_t octave){
  const uint16_t noteFrequencyBase[12] = {
  //  1      2      3      4      5      6     7       8     9       10    11      12
  //  C      C#     D      Eb     E      F     F#      G     G#      A     Bb      B
     4186,  4435,  4699,  4978,  5274,  5588,  5920,  6272,  6645,  7040,  7459,  7902
  };
  if(octave > 8 || note >= NOTE_MAX){
    return 0;
  }
  double noteFreq =  (double)noteFrequencyBase[note] / (double)(1 << (8-octave));
  return noteFreq;
}

uint8_t noteNum = 0;

note_t noteConvertor(uint8_t note) {
  note_t temp;
  if (note < 7) {
    if (note == 0) temp = NOTE_MAX;
    else if (note == 1) temp = NOTE_C;
    else if (note == 2) temp = NOTE_Cs;
    else if (note == 3) temp = NOTE_D;
    else if (note == 4) temp = NOTE_Eb;
    else if (note == 5) temp = NOTE_E;
    else if (note == 6) temp = NOTE_F;
  } else {
    if (note == 7) temp = NOTE_Fs;
    else if (note == 8) temp = NOTE_G;
    else if (note == 9) temp = NOTE_Gs;
    else if (note == 10) temp = NOTE_A;
    else if (note == 11) temp = NOTE_Bb;
    else if (note == 12) temp = NOTE_B;
  }
  return temp;
}

// --------------- note setup ----------------------------------

#include <EEPROM.h>
#include <WiFiEsp.h>

#include <SoftwareSerial.h> 
#define rxPin 3 
#define txPin 2 
SoftwareSerial esp01(txPin, rxPin); // SoftwareSerial NAME(TX, RX);

const char ssid[] = "SK_WiFiGIGA40F7";    // your network SSID (name)
const char pass[] = "1712037218";         // your network password
int status = WL_IDLE_STATUS;              // the Wifi radio's status

const char ssid_AP[] = "esp01_AP";        // AP Name
const char pass_AP[] = "1234test";        // AP password

WiFiEspServer server(80);

int beepPin = 10;

void setup() {
  Serial.begin(9600);
  esp01.begin(9600);   //와이파이 시리얼
  WiFi.init(&esp01);   // initialize ESP module
  while ( status != WL_CONNECTED) {   // 약 10초동안 wifi 연결 시도
    Serial.print(F("Attempting to connect to WPA SSID: "));
    Serial.println(ssid);
    status = WiFi.begin(ssid, pass);    // Connect to WPA/WPA2 network
    IPAddress ip = WiFi.localIP();
    Serial.print("IP Address: ");
    Serial.println(ip);
  }
  WiFi.beginAP(ssid_AP, 10, pass_AP, ENC_TYPE_WPA2_PSK, false); // 10:채널, false: softAP와 station 동시 사용모드
  IPAddress ap = WiFi.localIP();
  Serial.print("AP Address: ");
  Serial.println(ap);
  server.begin();
}

uint8_t Tempo = 120; // 1, 1박자 = 1000/(tempoRatio), 비트수 = 2 = 2000
uint8_t tempoRatio = Tempo/60;
uint8_t octaveTemp = 4; // 조표 변화시 옥타브 변경에 사용

unsigned long int beepTime = 0;

uint16_t noteTTL = 10; // 2,3
const uint8_t noteSt = 4;
uint16_t octSt = 18;   // noteTTL+noteSt
uint16_t durSt = 29;   // noteTTL+octSt+1

bool eepromMelody = false;
uint8_t eepromDur = 0;

void wifi() {
  WiFiEspClient client = server.available();  // listen for incoming clients
  if (client) {                               // if you get a client,
    while (client.connected()) {              // loop while the client's connected
      if (client.available()) {               // if there's bytes to read from the client,
        String line = client.readStringUntil('\n');
        String temp = line.substring(5, 9); // "GET /%%F0"
        if (temp == F("%%F0")) { // GET /%%F0tempo,notenum%(note,...)%(oct,...)%(dur,...)\n
          temp = line.substring(9, 12);
          if (temp == F("%S%")) {
            eepromMelody = false;
            noTone(beepPin);  
          } 
          else if (temp == F("%P%")) {
            eepromMelody = true;
            noteNum = 0; beepTime = 0;
            Tempo = EEPROM.read(1);
            EEPROM.get(2, noteTTL); //EEPROM.get(2);
            octSt = noteTTL+noteSt;  
            durSt = noteTTL+octSt+1; 
            tempoRatio = Tempo/60;
            eepromDur = EEPROM.read(durSt+noteNum);
          } 
          else {
            uint8_t value = 0;
            uint16_t indexSt = 9;
            uint16_t indexEd = line.indexOf(F(","));
            String temp = line.substring(indexSt, indexEd);
            value = temp.toInt();
            Tempo = value;    
            indexSt = line.indexOf(F("%"),indexEd);
            temp = line.substring(indexEd+1,indexSt);
            value = temp.toInt();
            noteTTL = value;
            octSt = noteTTL+noteSt;  
            durSt = noteTTL+octSt+1; 
            EEPROM.write(1,Tempo);
            EEPROM.put(2,noteTTL);
            uint8_t indexDel = line.lastIndexOf(F("%"));
            indexSt = indexDel+1;
            for (int i = 0; i < noteTTL+1; i++) {
              indexEd = line.indexOf(F(","), indexSt);
              temp = line.substring(indexSt, indexEd);
              value = temp.toInt();
              EEPROM.write(durSt+i,value);
              indexSt = indexEd+1;
            }
            line.remove(indexDel);
            indexDel = line.lastIndexOf(F("%"));
            indexSt = indexDel+1;
            for (int i = 0; i < noteTTL+1; i++) {
              indexEd = line.indexOf(F(","), indexSt);
              temp = line.substring(indexSt, indexEd);
              value = temp.toInt();
              EEPROM.write(octSt+i,value);
              indexSt = indexEd+1;
            }
            line.remove(indexDel);
            indexDel = line.lastIndexOf(F("%"));
            indexSt = indexDel+1;
            for (int i = 0; i < noteTTL; i++) {
              indexEd = line.indexOf(F(","), indexSt);
              temp = line.substring(indexSt, indexEd);
              value = temp.toInt();
              EEPROM.write(noteSt+i,value);
              indexSt = indexEd+1;
            }
            eepromMelody = true;
          }
          client.flush();
          client.println(F("HTTP/1.1 200 OK\nContent-type:text/html\nConnection: close\r\n"));
          client.println(F("%M%"));
        } else {
          client.flush();
          client.println(F("HTTP/1.1 200 OK\nContent-type:text/html\nConnection: close\r\n"));
          client.println(F("%C%"));
        }
        break;
      }
    }
    client.stop();
    Serial.println(F("Client disconnected"));
  }
}

void loop() {
  wifi();
  if (eepromMelody) {
    if (millis() - beepTime >= eepromDur*(1000/(4*tempoRatio))) {
      beepTime = millis();
      noTone(beepPin);   
      octaveTemp = EEPROM.read(octSt+noteNum);
      note_t KeyNote = noteConvertor(EEPROM.read(noteSt+noteNum));
      tone(beepPin, ledcWriteNote(KeyNote, octaveTemp));
      noteNum++;
      if (noteNum == noteTTL+1) { // 초기화
        noTone(beepPin);    //  noTone() - ledcWrite(uint8_t channel, uint32_t duty);
        noteNum = 0; 
      }
      eepromDur = EEPROM.read(durSt+noteNum);
    }
  }
  if(Serial.available() > 0){
    String temp = Serial.readStringUntil('\n');
    Serial.println(temp);
    if (temp == "1") {
      eepromMelody = !eepromMelody;
      noteNum = 0; beepTime = 0;
      if (eepromMelody) {
        Tempo = EEPROM.read(1);
        EEPROM.get(2, noteTTL); 
        octSt = noteTTL+noteSt;  
        durSt = noteTTL+octSt+1; 
        tempoRatio = Tempo/60;
        eepromDur = EEPROM.read(durSt+noteNum);
      } else {
        noTone(beepPin);  
      }
    }
    else if (temp == "2") {
      eepromMelody = false;
      noTone(beepPin);  
    }
  }
}
